@@ -465,8 +465,9 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
             "sub_name": share["id"],
             "size": size,
             "namespace_isolated": True,
-            "mode": self._cephfs_volume_mode,
+            "mode": self._cephfs_volume_mode
         }
+
         if share['share_group_id'] is not None:
             argdict.update({"group_name": share["share_group_id"]})
 
@@ -540,8 +541,14 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
             share_server=share_server)
 
     def ensure_share(self, context, share, share_server=None):
-        # Creation is idempotent
-        return self.create_share(context, share, share_server)
+        try:
+            export_location = self._get_export_locations(share)
+        except exception.ShareBackendException as e:
+            if 'does not exist' in str(e).lower():
+                raise exception.ShareResourceNotFound(share_id=share['id'])
+            raise
+
+        return export_location
 
     def extend_share(self, share, new_size, share_server=None):
         # resize FS subvolume/share
@@ -595,7 +602,7 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
         argdict = {
             "vol_name": self.volname,
             "sub_name": snapshot["share_id"],
-            "snap_name": "_".join([snapshot["snapshot_id"], snapshot["id"]]),
+            "snap_name": snapshot["snapshot_id"],
         }
 
         rados_command(
@@ -605,13 +612,24 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
         # delete a FS snapshot
         LOG.debug("[%(be)s]: delete_snapshot: snapshot=%(id)s.",
                   {"be": self.backend_name, "id": snapshot['id']})
-        argdict = {
+
+        # FIXME(vkmc) remove this in CC (next tick) release.
+        legacy_snap_name = "_".join([snapshot["snapshot_id"], snapshot["id"]])
+
+        argdict_legacy = {
             "vol_name": self.volname,
             "sub_name": snapshot["share_id"],
-            "snap_name": '_'.join([snapshot['snapshot_id'], snapshot['id']]),
+            "snap_name": legacy_snap_name,
             "force": True,
         }
 
+        # try removing snapshot using legacy naming
+        rados_command(
+            self.rados_client, "fs subvolume snapshot rm", argdict_legacy)
+
+        # in case it's a snapshot with new naming, retry remove with new name
+        argdict = argdict_legacy.copy()
+        argdict.update({"snap_name": snapshot["snapshot_id"]})
         rados_command(self.rados_client, "fs subvolume snapshot rm", argdict)
 
     def create_share_group(self, context, sg_dict, share_server=None):
@@ -734,7 +752,7 @@ class CephFSDriver(driver.ExecuteMixin, driver.GaneshaMixin,
         argdict = {
             "vol_name": self.volname,
             "sub_name": parent_share["id"],
-            "snap_name": '_'.join([snapshot["snapshot_id"], snapshot["id"]]),
+            "snap_name": snapshot["snapshot_id"],
             "target_sub_name": share["id"]
         }
         if share['share_group_id'] is not None:
@@ -1112,7 +1130,8 @@ class NFSProtocolHelper(NFSProtocolHelperMixin, ganesha.GaneshaNASHelper2):
         return {
             'Name': 'Ceph',
             'User_Id': ceph_auth_id,
-            'Secret_Access_Key': auth_result
+            'Secret_Access_Key': auth_result,
+            'Filesystem': self.volname
         }
 
     def _cleanup_fsal_hook(self, base, share, access):
@@ -1196,6 +1215,13 @@ class NFSClusterProtocolHelper(NFSProtocolHelperMixin, ganesha.NASHelperBase):
         else:
             export_ips.append(vip)
 
+        # there are no export IPs, there are no NFS servers we can use
+        if not export_ips:
+            msg = _("There are no NFS servers available to use. "
+                    "Please check the health of your Ceph cluster "
+                    "and restart the manila share service.")
+            raise exception.ShareBackendException(msg=msg)
+
         return export_ips
 
     def check_for_setup_error(self):
