@@ -273,29 +273,26 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
                     msg=_("Could not destroy '%s' dataset, "
                           "because it had opened files.") % name)
 
-        # NOTE(vponomaryov): Now, when no file usages and mounts of dataset
-        # exist, destroy dataset.
-        try:
+        @utils.retry(retry_param=exception.ProcessExecutionError, retries=10)
+        def _zfs_destroy_with_retry():
+            """Retry destroying dataset ten times with exponential backoff."""
+            # NOTE(bswartz): There appears to be a bug in ZFS when creating and
+            # destroying datasets concurrently where the filesystem remains
+            # mounted even though ZFS thinks it's unmounted. The most reliable
+            # workaround I've found is to force the unmount, then attempt the
+            # destroy, with short pauses around the unmount. (See bug#1546723)
+            try:
+                self.execute('sudo', 'umount', mountpoint)
+            except exception.ProcessExecutionError:
+                # Ignore failed umount, it's normal
+                pass
+            time.sleep(2)
+
+            # NOTE(vponomaryov): Now, when no file usages and mounts of dataset
+            # exist, destroy dataset.
             self.zfs('destroy', '-f', name)
-            return
-        except exception.ProcessExecutionError:
-            LOG.info("Failed to destroy ZFS dataset, retrying one time")
 
-        # NOTE(bswartz): There appears to be a bug in ZFS when creating and
-        # destroying datasets concurrently where the filesystem remains mounted
-        # even though ZFS thinks it's unmounted. The most reliable workaround
-        # I've found is to force the unmount, then retry the destroy, with
-        # short pauses around the unmount.
-        time.sleep(1)
-        try:
-            self.execute('sudo', 'umount', mountpoint)
-        except exception.ProcessExecutionError:
-            # Ignore failed umount, it's normal
-            pass
-        time.sleep(1)
-
-        # This time the destroy is expected to succeed.
-        self.zfs('destroy', '-f', name)
+        _zfs_destroy_with_retry()
 
     def _setup_helpers(self):
         """Setups share helper for ZFS backend."""
@@ -361,6 +358,12 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
                 'free_capacity_gb': float(free_size),
                 'reserved_percentage':
                     self.configuration.reserved_share_percentage,
+                'reserved_snapshot_percentage': (
+                    self.configuration.reserved_share_from_snapshot_percentage
+                    or self.configuration.reserved_share_percentage),
+                'reserved_share_extend_percentage': (
+                    self.configuration.reserved_share_extend_percentage
+                    or self.configuration.reserved_share_percentage),
             }
             pool.update(self.common_capabilities)
             if self.configuration.replication_domain:
@@ -375,6 +378,12 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
             'storage_protocol': 'NFS',
             'reserved_percentage':
                 self.configuration.reserved_share_percentage,
+            'reserved_snapshot_percentage': (
+                self.configuration.reserved_share_from_snapshot_percentage
+                or self.configuration.reserved_share_percentage),
+            'reserved_share_extend_percentage': (
+                self.configuration.reserved_share_extend_percentage
+                or self.configuration.reserved_share_percentage),
             'snapshot_support': True,
             'create_share_from_snapshot_support': True,
             'driver_name': 'ZFS',
@@ -580,33 +589,50 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
 
     @ensure_share_server_not_provided
     def create_share_from_snapshot(self, context, share, snapshot,
-                                   share_server=None):
+                                   share_server=None, parent_share=None):
         """Is called to create a share from snapshot."""
+        src_backend_name = share_utils.extract_host(
+            snapshot.share_instance['host'], level='backend_name'
+        )
+        src_snapshot_name = self._get_saved_snapshot_name(snapshot)
         dataset_name = self._get_dataset_name(share)
-        ssh_cmd = '%(username)s@%(host)s' % {
+
+        dst_backend_ssh_cmd = '%(username)s@%(host)s' % {
             'username': self.configuration.zfs_ssh_username,
             'host': self.service_ip,
         }
-        pool_name = share_utils.extract_host(share['host'], level='pool')
+
+        dst_backend_pool_name = share_utils.extract_host(share['host'],
+                                                         level='pool')
         options = self._get_dataset_creation_options(share, is_readonly=False)
+
         self.private_storage.update(
             share['id'], {
                 'entity_type': 'share',
                 'dataset_name': dataset_name,
-                'ssh_cmd': ssh_cmd,  # used in replication
-                'pool_name': pool_name,  # used in replication
+                'ssh_cmd': dst_backend_ssh_cmd,  # used in replication
+                'pool_name': dst_backend_pool_name,  # used in replication
                 'used_options': options,
             }
         )
-        snapshot_name = self._get_saved_snapshot_name(snapshot)
 
+        # NOTE(andrebeltrami): Implementing the support for create share
+        # from snapshot in different backends in different hosts
+        src_config = get_backend_configuration(src_backend_name)
+        src_backend_ssh_cmd = '%(username)s@%(host)s' % {
+            'username': src_config.zfs_ssh_username,
+            'host': src_config.zfs_service_ip,
+        }
         self.execute(
             # NOTE(vponomaryov): SSH is used as workaround for 'execute'
-            # implementation restriction that does not support usage of '|'.
-            'ssh', ssh_cmd,
-            'sudo', 'zfs', 'send', '-vD', snapshot_name, '|',
+            # implementation restriction that does not support usage
+            # of '|'.
+            'ssh', src_backend_ssh_cmd,
+            'sudo', 'zfs', 'send', '-vD', src_snapshot_name, '|',
+            'ssh', dst_backend_ssh_cmd,
             'sudo', 'zfs', 'receive', '-v', dataset_name,
         )
+
         # Apply options based on used share type that may differ from
         # one used for original share.
         for option in options:
@@ -615,7 +641,7 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
         # Delete with retry as right after creation it may be temporary busy.
         self.execute_with_retry(
             'sudo', 'zfs', 'destroy',
-            dataset_name + '@' + snapshot_name.split('@')[-1])
+            dataset_name + '@' + src_snapshot_name.split('@')[-1])
 
         return self._get_share_helper(
             share['share_proto']).create_exports(dataset_name)
@@ -724,15 +750,6 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
             "username": self.configuration.zfs_ssh_username,
             "host": self.service_ip,
         }
-        self.private_storage.update(
-            share["id"], {
-                "entity_type": "share",
-                "dataset_name": new_dataset_name,
-                "ssh_cmd": ssh_cmd,  # used in replication
-                "pool_name": actual_pool_name,  # used in replication
-                "used_options": " ".join(options),
-            }
-        )
 
         # Perform checks on requested dataset
         if actual_pool_name != scheduled_pool_name:
@@ -760,13 +777,25 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
                     "dataset": old_dataset_name,
                     "zpool": actual_pool_name})
 
-        # Rename dataset
-        out, err = self.execute("sudo", "mount")
-        if "%s " % old_dataset_name in out:
-            self.zfs_with_retry("umount", "-f", old_dataset_name)
-            time.sleep(1)
+        # Unmount the dataset before attempting to rename and mount
+        try:
+            self._unmount_share_with_retry(old_dataset_name)
+        except exception.ZFSonLinuxException:
+            msg = _("Unable to unmount share before renaming and re-mounting.")
+            raise exception.ZFSonLinuxException(message=msg)
+
+        # Rename the dataset and mount with new name
         self.zfs_with_retry("rename", old_dataset_name, new_dataset_name)
-        self.zfs("mount", new_dataset_name)
+
+        try:
+            self.zfs("mount", new_dataset_name)
+        except exception.ProcessExecutionError:
+            # Workaround for bug/1785180
+            out, err = self.zfs("mount")
+            mounted = any([new_dataset_name in mountedfs
+                           for mountedfs in out.splitlines()])
+            if not mounted:
+                raise
 
         # Apply options to dataset
         for option in options:
@@ -776,6 +805,16 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
         export_locations = self._get_share_helper(
             share["share_proto"]).get_exports(new_dataset_name)
 
+        self.private_storage.update(
+            share["id"], {
+                "entity_type": "share",
+                "dataset_name": new_dataset_name,
+                "ssh_cmd": ssh_cmd,  # used in replication
+                "pool_name": actual_pool_name,  # used in replication
+                "used_options": " ".join(options),
+            }
+        )
+
         return {"size": share["size"], "export_locations": export_locations}
 
     def unmanage(self, share):
@@ -836,6 +875,17 @@ class ZFSonLinuxShareDriver(zfs_utils.ExecuteMixin, driver.ShareDriver):
         """Unmanage dataset snapshot."""
         self.private_storage.delete(snapshot_instance["snapshot_id"])
 
+    @utils.retry(retry_param=exception.ZFSonLinuxException, retries=10)
+    def _unmount_share_with_retry(self, share_name):
+        out, err = self.execute("sudo", "mount")
+        if "%s " % share_name not in out:
+            return
+        self.zfs_with_retry("umount", "-f", share_name)
+        out, err = self.execute("sudo", "mount")
+        if "%s " % share_name in out:
+            raise exception.ZFSonLinuxException(
+                _("Unable to unmount dataset %s"), share_name)
+
     def _get_replication_snapshot_prefix(self, replica):
         """Returns replica-based snapshot prefix."""
         replication_snapshot_prefix = "%s_%s" % (
