@@ -252,6 +252,8 @@ class GlusterfsNativeShareDriverTestCase(test.TestCase):
             'driver_version': '1.1',
             'storage_protocol': 'glusterfs',
             'reserved_percentage': 0,
+            'reserved_snapshot_percentage': 0,
+            'reserved_share_extend_percentage': 0,
             'qos': False,
             'total_capacity_gb': 'unknown',
             'free_capacity_gb': 'unknown',
@@ -269,6 +271,8 @@ class GlusterfsNativeShareDriverTestCase(test.TestCase):
             'ipv4_support': True,
             'ipv6_support': False,
             'security_service_update_support': False,
+            'share_server_multiple_subnet_support': False,
+            'network_allocation_update_support': False,
         }
         self.assertEqual(test_data, self._driver._stats)
 
