@@ -16,7 +16,6 @@
 #    under the License.
 
 import ddt
-import six
 
 from manila import exception
 from manila import test
@@ -46,18 +45,18 @@ class ManilaExceptionTestCase(test.TestCase):
             message = "default message"
 
         exc = FakeManilaException()
-        self.assertEqual('default message', six.text_type(exc))
+        self.assertEqual('default message', str(exc))
 
     def test_error_msg(self):
         self.assertEqual('test',
-                         six.text_type(exception.ManilaException('test')))
+                         str(exception.ManilaException('test')))
 
     def test_default_error_msg_with_kwargs(self):
         class FakeManilaException(exception.ManilaException):
             message = "default message: %(code)s"
 
         exc = FakeManilaException(code=500)
-        self.assertEqual('default message: 500', six.text_type(exc))
+        self.assertEqual('default message: 500', str(exc))
 
     def test_error_msg_exception_with_kwargs(self):
         # NOTE(dprince): disable format errors for this test
@@ -68,7 +67,7 @@ class ManilaExceptionTestCase(test.TestCase):
 
         exc = FakeManilaException(code=500)
         self.assertEqual('default message: %(misspelled_code)s',
-                         six.text_type(exc))
+                         str(exc))
 
     def test_default_error_code(self):
         class FakeManilaException(exception.ManilaException):
@@ -445,9 +444,9 @@ class ManilaExceptionResponseCode404(test.TestCase):
         self.assertEqual(404, e.code)
         self.assertIn(snapshot_id, e.msg)
 
-    def test_share_metadata_not_found(self):
-        # verify response code for exception.ShareMetadataNotFound
-        e = exception.ShareMetadataNotFound()
+    def test_metadata_item_not_found(self):
+        # verify response code for exception.MetadataItemNotFound
+        e = exception.MetadataItemNotFound()
         self.assertEqual(404, e.code)
 
     def test_security_service_not_found(self):
@@ -603,3 +602,10 @@ class ManilaExceptionResponseCode413(test.TestCase):
         # verify response code for exception.PortLimitExceeded
         e = exception.PortLimitExceeded()
         self.assertEqual(413, e.code)
+
+    def test_per_share_limit_exceeded(self):
+        # verify response code for exception.ShareSizeExceedsLimit
+        size = 779  # amount of share size
+        limit = 775  # amount of allowed share size limit
+        e = exception.ShareSizeExceedsLimit(size=size, limit=limit)
+        self.assertEqual(413, e.code)
